/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using NAnt.Core.Types;

namespace TopCoder.Test.Framework.Servers
{
    /// <summary>
    /// <p>
    /// This class is derived from Server class and is the base class
    /// for all classes representing data server to be used while running tests.
    /// This class is a base for another component which will provide
    /// testing ability for database servers.
    /// TestCase classes should also be provided by that component.
    /// </p>
    /// <p>
    /// The supported options are:<br/>
    /// DB_DRIVER - specifies the name of the database driver to use for connecting to database.<br/>
    /// DB_CONNECTION_STR - specifies the database connection string to use for connecting to database.<br/>
    /// SQL_INIT_SCRIPT - specifies  the name of file, containing a SQL script used to init the data.<br/>
    /// SQL_CLEANUP_SCRIPT - specifies  the name of file, containing a SQL script used to clean-up the data.<br/>
    ///
    /// </p>
    ///
    /// <p>
    /// The parent class is mutable, so this class is mutable, i.e. not thread-safe.
    /// </p>
    /// </summary>
    ///
    /// <author>real_vg</author>
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    public abstract class DatabaseServer : Server
    {

        /// <summary>
        /// This string constanst represents the name (key) of the option, the derived classes
        /// should use for getting the name of the database driver to use for connecting to database.
        /// Note, that the derived classes can treat options in any way they like to do.
        /// </summary>
        public static readonly string DB_DRIVER = "db_driver";

        /// <summary>
        /// This string constanst represents the name (key) of the option, the derived classes
        /// should use for getting the database connection string to use for connecting to database.
        /// Note, that the derived classes can treat options in any way they like to do.
        /// </summary>
        public static readonly string DB_CONNECTION_STR = "db_connection_str";

        /// <summary>
        /// This string constanst represents the name (key) of the option, the derived classes
        /// should use for getting the name of file, containing a SQL script used to init the data.
        /// Note, that the derived classes can treat options in any way they like to do.
        /// The most common way of use of this options will be
        /// executing the specified script in the InitData() method.
        /// </summary>
        public static readonly string SQL_INIT_SCRIPT = "sql_init_script";

        /// <summary>
        /// This string constanst represents the name (key) of the option, the derived classes
        /// should use for getting the name of file, containing a SQL script used to clean-up the data.
        /// Note, that the derived classes can treat options in any way they like to do.
        /// The most common way of use of this options will be
        /// executing the specified script in the CleanUpData() method.
        /// </summary>
        public static readonly string SQL_CLEANUP_SCRIPT = "sql_cleanup_script";

        /// <summary>
        /// This property provides the read access to the option, the derived classes
        /// should use for getting the name of the database driver to use for connecting to database.
        /// Note, that the derived classes can treat options in any way they like to do.
        /// </summary>
        ///
        /// <value>The name of the database driver to use for connecting to database.</value>
        /// <exception cref="ServerException">If db_driver option is missing.</exception>
        public string DatabaseDriver
        {
            get
            {
                return GetOption(DB_DRIVER);
            }
        }

        /// <summary>
        /// This property provides the read access to the option, the derived classes
        /// should use for getting the database connection string to use for connecting to database.
        /// Note, that the derived classes can treat options in any way they like to do.
        /// </summary>
        ///
        /// <value>The database connection string to use for connecting to database.</value>
        /// <exception cref="ServerException">If db_connection_str option is missing.</exception>
        public string DatabaseConnectionString
        {
            get
            {
                return GetOption(DB_CONNECTION_STR);
            }
        }


        /// <summary>
        /// This property provides the read access to the option, the derived classes
        /// should use for getting the name of file containing an SQL script used to init the data.
        /// Note, that the derived classes can treat options in any way they like to do.
        /// The most common way of use of this options will be
        /// executing the specified script in the InitData() method.
        /// </summary>
        ///
        /// <value>The name of file containing an SQL script used to init the data.</value>
        /// <exception cref="ServerException">If sql_init_script option is missing.</exception>
        public string SqlInitScript
        {
            get
            {
                return GetOption(SQL_INIT_SCRIPT);
            }
        }


        /// <summary>
        /// This property provides the read access to the option, the derived classes
        /// should use for getting the name of file containing an SQL script used to clean up the data.
        /// Note, that the derived classes can treat options in any way they like to do.
        /// The most common way of use of this options will be
        /// executing the specified script in the CleanUpData() method.
        /// </summary>
        ///
        /// <value>The name of file containing an SQL script used to clean up the data.</value>
        /// <exception cref="ServerException">If sql_cleanup_script option is missing.</exception>
        public string SqlCleanUpScript
        {
            get
            {
                return GetOption(SQL_CLEANUP_SCRIPT);
            }
        }




        /// <summary>
        /// <p>
        /// Creates an instance of class.
        /// </p>
        /// </summary>
        protected DatabaseServer()
            : base()
        {
        }
    }
}
