/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using System;
using NUnit.Framework;
using NAnt.Core.Types;


namespace TopCoder.Test.Framework.Servers.Web
{

    /// <summary>
    /// Unit test class for <c>IISServer</c> class.
    /// </summary>
    ///
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    [TestFixture]
    public class IISServerTest
    {

        /// <summary>
        /// Private <c>IISServer</c> variable used for test.
        /// </summary>
        private IISServer server = null;

        /// <summary>
        /// Sets up the test environment. A valid <c>IISServer</c> object is created.
        /// </summary>
        [SetUp]
        protected void SetUp()
        {
            server = new IISServer();
        }

        /// <summary>
        /// Cleans up the test environment. The <c>IISServer</c> object is disposed.
        /// </summary>
        [TearDown]
        protected void TearDown()
        {
            TestFrameworkTestHelper.StopService();
            TestFrameworkTestHelper.DeleteFile("deploy/HelloWorld.aspx");
            TestFrameworkTestHelper.DeleteFile("deploy/subdir/HelloWorld.aspx");
        }

        /// <summary>
        /// Tests accuracy of constructor <c>IISServer()</c> of <c>IISServer</c> class.
        /// The fields should be initialized properly.
        /// </summary>
        [Test]
        public void TestIISServerAccuracy()
        {
            Assert.IsTrue(server.StartServer, "StartServer should be set as true.");
            Assert.IsTrue(server.InitServer, "InitServer should be set as true.");
            Assert.IsNull(server.Options, "Options should be set as null.");
            Assert.IsNull(server.Credentials, "Credentials should be set as null.");
            Assert.IsNull(server.SourceFiles, "SourceFiles should be set as null.");
        }

        /// <summary>
        /// Tests inheritance of constructor <c>IISServer()</c> of <c>IISServer</c> class.
        /// <c>IISServer</c> class should be inherited from <c>ApplicationServer</c> class.
        /// </summary>
        [Test]
        public void TestIISServerInheritance()
        {
            Assert.IsTrue(((object) server) is ApplicationServer,
                "IISServer class should be inherited from ApplicationServer class.");
        }

        /// <summary>
        /// Tests accuracy of constant <c>SERVICE_NAME</c> of <c>IISServer</c> class.
        /// The fields should be initialized properly.
        /// </summary>
        [Test]
        public void TestConstantServiceNameAccuracy()
        {
            Assert.AreEqual("service_name", IISServer.SERVICE_NAME,
                "SERVICE_NAME constant should be correct.");
        }

        /// <summary>
        /// Tests the StartUp method with invalid service name.
        /// ServerException should be thrown
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestStartUpInvalidServiceName()
        {
            TestFrameworkTestHelper.LoadServer("invalid_server1.xml").StartUp();
        }

        /// <summary>
        /// Tests the StartUp method with invalid deploy dir.
        /// ServerException should be thrown
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestStartUpInvalidDeployPath()
        {
            TestFrameworkTestHelper.LoadServer("invalid_server2.xml").StartUp();
        }

        /// <summary>
        /// Tests the StartUp method with invalid timeout.
        /// ServerException should be thrown
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestStartUpInvalidTimeout1()
        {
            TestFrameworkTestHelper.LoadServer("invalid_server3.xml").StartUp();
        }

        /// <summary>
        /// Tests the StartUp method with invalid timeout.
        /// ServerException should be thrown
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestStartUpInvalidTimeout2()
        {
            TestFrameworkTestHelper.LoadServer("invalid_server4.xml").StartUp();
        }

        /// <summary>
        /// Tests the StartUp method with invalid timeout.
        /// ServerException should be thrown
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestStartUpInvalidTimeout3()
        {
            TestFrameworkTestHelper.LoadServer("invalid_server5.xml").StartUp();
        }

        /// <summary>
        /// Tests the StartUp method.
        /// ServerException should be thrown if the service is already started.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestStartUpStartedService()
        {

            TestFrameworkTestHelper.StartService();

            TestFrameworkTestHelper.LoadServer("iisserver1.xml").StartUp();

        }

        /// <summary>
        /// Tests the StartUp method.
        /// IIS service should be started.
        /// </summary>
        [Test]
        public void TestStartUpAccuracy1()
        {
            TestFrameworkTestHelper.LoadServer("iisserver1.xml").StartUp();

            TestFrameworkTestHelper.StartService();

            Assert.IsTrue(TestFrameworkTestHelper.IsServiceStarted(), "service should be started.");
            Assert.IsFalse(TestFrameworkTestHelper.FileDeployed("HelloWorld.aspx"), "file should not be deployed.");
            Assert.IsFalse(TestFrameworkTestHelper.FileDeployed("subdir/HelloWorld.aspx"), "file should not be deployed.");


        }


        /// <summary>
        /// Tests the StartUp method.
        /// Files should be deployed.
        /// </summary>
        [Test]
        public void TestStartUpAccuracy2()
        {
            TestFrameworkTestHelper.LoadServer("iisserver2.xml").StartUp();

            Assert.IsTrue(TestFrameworkTestHelper.IsServiceStarted(), "service should be started.");
            Assert.IsTrue(TestFrameworkTestHelper.FileDeployed("HelloWorld.aspx"), "file should be deployed.");
            Assert.IsTrue(TestFrameworkTestHelper.FileDeployed("subdir/HelloWorld.aspx"), "file should be deployed.");
        }

        /// <summary>
        /// Tests the StartUp method using default values.
        /// IIS service should be started.
        /// Files should be deployed.
        /// </summary>
        [Test]
        public void TestStartUpAccuracy3()
        {
            TestFrameworkTestHelper.LoadServer("iisserver3.xml").StartUp();

            Assert.IsTrue(TestFrameworkTestHelper.IsServiceStarted(), "service should be started.");
            Assert.IsTrue(TestFrameworkTestHelper.FileDeployed("HelloWorld.aspx"), "file should be deployed.");
            Assert.IsTrue(TestFrameworkTestHelper.FileDeployed("subdir/HelloWorld.aspx"), "file should be deployed.");
        }

        /// <summary>
        /// Tests the ShutDown method with invalid service name.
        /// ServerException should be thrown
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestShutDownInvalidServiceName()
        {
            TestFrameworkTestHelper.LoadServer("invalid_server1.xml").ShutDown();
        }

        /// <summary>
        /// Tests the ShutDown method with invalid deploy dir.
        /// ServerException should be thrown
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestShutDownInvalidDeployPath()
        {
            TestFrameworkTestHelper.LoadServer("invalid_server2.xml").ShutDown();
        }

        /// <summary>
        /// Tests the ShutDown method with invalid timeout.
        /// ServerException should be thrown
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestShutDownInvalidTimeout1()
        {
            TestFrameworkTestHelper.LoadServer("invalid_server3.xml").ShutDown();
        }

        /// <summary>
        /// Tests the ShutDown method with invalid timeout.
        /// ServerException should be thrown
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestShutDownInvalidTimeout2()
        {
            TestFrameworkTestHelper.LoadServer("invalid_server4.xml").ShutDown();
        }

        /// <summary>
        /// Tests the ShutDown method with invalid timeout.
        /// ServerException should be thrown
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestShutDownInvalidTimeout3()
        {
            TestFrameworkTestHelper.LoadServer("invalid_server5.xml").ShutDown();
        }

        /// <summary>
        /// Tests the ShutDown method.
        /// ServerException should be thrown if the service is already shutdown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerException))]
        public void TestShutDownStartedService()
        {

            TestFrameworkTestHelper.LoadServer("iisserver1.xml").ShutDown();

        }

        /// <summary>
        /// Tests the ShutDown method.
        /// IIS service should be shotdown.
        /// </summary>
        [Test]
        public void TestShutDownAccuracy1()
        {
            TestFrameworkTestHelper.StartService();

            // IIS service should be shotdown if startserver is true.
            TestFrameworkTestHelper.LoadServer("iisserver1.xml").ShutDown();

            Assert.IsFalse(TestFrameworkTestHelper.IsServiceStarted(), "service should be started.");

        }


        /// <summary>
        /// Tests the ShutDown method.
        /// IIS service should be shotdown regardless startserver is false.
        /// </summary>
        [Test]
        public void TestShutDownAccuracy2()
        {
            TestFrameworkTestHelper.StartService();

            // IIS service should shotdown
            TestFrameworkTestHelper.LoadServer("iisserver2.xml").ShutDown();

            Assert.IsFalse(TestFrameworkTestHelper.IsServiceStarted(), "service should be shutdown");
        }

        /// <summary>
        /// Tests the ShutDown method.
        /// IIS service should be shotdown if startserver is false.
        /// </summary>
        [Test]
        public void TestShutDownAccuracy3()
        {
            // deploy the files and then undeploy
            server = (IISServer) TestFrameworkTestHelper.LoadServer("iisserver2.xml");
            server.StartUp();

            server.ShutDown();

            // files should be deleted.
            Assert.IsFalse(TestFrameworkTestHelper.FileDeployed("HelloWorld.aspx"), "file should be undeployed.");
            Assert.IsFalse(TestFrameworkTestHelper.FileDeployed("subdir/HelloWorld.aspx"), "file should be undeployed.");
        }

        /// <summary>
        /// Tests the ShutDown method..
        /// Files should not be undeployed if initserver is false.
        /// </summary>
        [Test]
        public void TestShutDownAccuracy4()
        {
            TestFrameworkTestHelper.LoadServer("iisserver2.xml").StartUp();

            // just stop the service, don't undeploy
            TestFrameworkTestHelper.LoadServer("iisserver1.xml").ShutDown();

            // service should be stopped, but the files should not be undeployed
            Assert.IsFalse(TestFrameworkTestHelper.IsServiceStarted(), "service should be shutdown.");
            Assert.IsTrue(TestFrameworkTestHelper.FileDeployed("HelloWorld.aspx"), "file should not be undeployed.");
            Assert.IsTrue(TestFrameworkTestHelper.FileDeployed("subdir/HelloWorld.aspx"),
                "file should not be undeployed.");
        }


        /// <summary>
        /// Tests the CleanUpData method.
        /// Nothing happens.
        /// </summary>
        [Test]
        public void TestCleanUpData()
        {
            // nothing happens
            TestFrameworkTestHelper.LoadServer("iisserver1.xml").CleanUpData();
        }

        /// <summary>
        /// Tests the InitData method.
        /// Nothing happens.
        /// </summary>
        [Test]
        public void TestInitData()
        {
            // nothing happens
            TestFrameworkTestHelper.LoadServer("iisserver2.xml").InitData();
        }

    }

}
